// fopcinjof.cc
#include <fopcinjof.h>
#include <Axis.h>

fopCinjof::fopCinjof(floatArray& filt, 
                   const char* axname0, const char* axname1,
                   const int    mylag0, const int   mylag1 ){
    Axis inax0(0,0,0,axname0,1,1,1);
    Axis inax1(0,0,0,axname1,1,1,1);    
    Axislist inaxis(2);
    inaxis.list[0]  = inax0; 
    inaxis.list[1]  = inax1; 
    outaxis = inaxis;
    lag0 = mylag0;
    lag1 = mylag1;
    filter = filt;
}
fopCinjof::fopCinjof(floatArray& filt, 
		   const char* axname0, const char* axname1,
                   const float  delta0, const float  delta1, 
                   const int    mylag0, const int    mylag1 ) {
    Axis inax0(0,delta0,0,axname0,1,0,1);
    Axis inax1(0,delta1,0,axname1,1,0,1);    
    Axislist inaxis(2);
    inaxis.list[0]  = inax0; 
    inaxis.list[1]  = inax1; 
    outaxis = inaxis;
    lag0 = mylag0;
    lag1 = mylag1;
    filter = filt;
}
fopCinjof::fopCinjof(const floatspace & filtsp, int usedelta,
                   const int   mylag0, const int   mylag1 ) {
    lag0 = mylag0;
    lag1 = mylag1;
    filter = getdata(filtsp);             // get the filter coefficients
    Axislist ax = getinfo(filtsp);	  // get the axis list.
    for( int i=2; i < ax.ndim ; i++ ){     // check input is 1D
	if( ax.list[i].length != 1 ){
	   cerr << "floatCinjof needs 2-D Space as input.\n";
	   exit(1);
	}
    }
    // Make an axis to be matched
    Axislist inax = Axislist(2);
    inax.list[0].name = ax.list[0].name; 
    inax.list[1].name = ax.list[1].name; 
    if( usedelta ){
      inax.list[0].delta =  ax.list[0].delta ; 
      inax.list[1].delta =  ax.list[1].delta ;
      }
    inaxis = Axislist( inax, 1, !usedelta, 1);
    outaxis = inaxis;
}
void fopCinjof::forwardwildtrans(Axislist & out,const Axislist & in ) const {
    out.list[0]=in.list[0];
    out.list[1]=in.list[1];
}
void fopCinjof::adjointwildtrans(Axislist & out,const Axislist & in ) const {
    out.list[0]=in.list[0];
    out.list[1]=in.list[1];
}
void fopCinjof::apply(floatArray & xx,floatArray & yy,int adj,
		     const Axislist & xal, const Axislist & yal) const {

    int nx0 = xal.list[0].length ; int nb0 = ::rows(filter);     
    int nx1 = xal.list[1].length ; int nb1 = ::cols(filter); 
    int ny0 = yal.list[0].length ;
    int ny1 = yal.list[1].length ;

    for (int b1 = 0       ; b1 < nb1     ; b1++) {
    for (int y1 = nb1-lag1; y1 < ny1-lag1; y1++) { int x1 = y1 - b1 + lag1;
    for (int b0 = 0       ; b0 < nb0     ; b0++) {
    for (int y0 = nb0-lag0; y0 < ny0-lag0; y0++) { int x0 = y0 - b0 + lag0;
       if (adj) {
	   xx(x0,x1) += filter(b0,b1) * yy(y0,y1);
       } 
       else   
       {
	   yy(y0,y1) += filter(b0,b1) * xx(x0,x1);
       }
					       }}}}
}

