// fopconv2.cc
#include <fopconv2.h>
#include <Axis.h>

fopConv2::fopConv2( floatArray & filt,const char * axisname) {
    Axis inax(0,0,0,axisname,1,1,1);      // Make an axis to be matched
    inaxis = (Axislist) inax;
    outaxis = inaxis;
    filter = filt;
}
fopConv2::fopConv2(floatArray & filt,const char *axisname, float delta) {
    Axis inax(0,delta,0,axisname,1,0,1);  // Make an axis to be matched
    inaxis  = (Axislist) inax;
    outaxis = inaxis;
    filter = filt;
}
fopConv2::fopConv2(const floatspace & filtsp, int usedelta) {
    Axislist ax = getinfo(filtsp);	  // get the axis list.
    filter = getdata(filtsp);             // get the filter coefficients
    for( int i=1; i< ax.ndim ; i++ ){     // check input is 1D
	if( ax.list[i].length != 1 ){
	   cerr << "floatConv2 needs 2-D Space as input.\n";
	   exit(1);
	}
    }
    // Make an axis to be matched
    Axis inax(0, ax.list[0].delta, 0,ax.list[0].name,1,!usedelta,1);
    inaxis = (Axislist) inax;
    outaxis = inaxis;
}
void fopConv2::forwardwildtrans(Axislist & out,const Axislist & in ) const {
    out.list[0]=in.list[0];
    out.list[0].length += ( ::rows(filter) - 1 );
}
void fopConv2::adjointwildtrans(Axislist & out,const Axislist & in ) const {
    out.list[0]=in.list[0];
    out.list[0].length -= ( ::rows(filter) - 1);
}
void fopConv2::apply(floatArray & x,floatArray & y,int adj,
		     const Axislist & xal, const Axislist & yal) const {
    for (int iy=0; iy<(::rows(filter)); iy++) {
	for (int ix=0; ix<xal.list[0].length; ix++) {
	    if (adj) {
		x(ix)    += y(iy+ix) * filter(iy);
	    } else {
		y(iy+ix) += x(ix)    * filter(iy);
	    }
	}
    }
}

