//   missif.cc : 1-D missing data interpolation with unknown filter
#include "missif.h"
floatspacearray missif(const floatspace & f, const floatspace & d, 
			                     int lag, int niter ) {
	floatspace data = d, filt = f;  
	floatArray ff   = filt.getarray();
        Axislist   dax  = data.getaxislist(); 
	Axislist   fax  = filt.getaxislist(); 
	floatspacearray x(2,1),resid;    
	x(0,0) = data;                          // array of data and filter
	x(1,0) = filt;         

	if( ff(lag - 1) == 0.)              
		seperr( "Error: filter(lag) = 0. \n");

	int * mask = new int[fax.list[0].length]; 
        for( int i=0; i < fax.list[0].length ; i++)             
                if ( i != lag-1 )
                        mask[i] = 1;            // mask for filter
	
	fopDiagonal mymask(2);                  // diagonal array of Mask op
        fopMask     dmask(dax.list[0],data); 
	fopMask     fmask(fax.list[0],mask);   
        mymask.set(0,dmask); 
        mymask.set(1,fmask); 
	
	for( int iter=0; iter < niter; 	iter++){    // outer nonlinear loop
	                                            // update Operators 
		fopContran * convf  = new fopContran(filt,0);
		fopContran * convd  = new fopContran(data,0);	
		fopArray   * myconv = new fopArray(1,2);
		myconv->set(0,0,*convf); 
		myconv->set(0,1,*convd);
	        fopChain   * myop   = new fopChain(*myconv, mymask); 

		resid = -(convf->Forward(data));    // make residual 
  		x = fophestenes(x, resid, *myop, 2);// Hestenes Linear Solver 
		data += x(0,0);                     // update unknowns
		filt += x(1,0);      
		
		delete convf; delete convd; delete myconv; delete myop;
	}
	delete mask;
	return x;
}
